/*
 * 
 * @author: Dave Siracusa, Jack Hart
 * @modified: Jack Hart
 * 
 */
package org.jboss.soa.esb.samples.quickstart.simplecbr;

import org.jboss.soa.esb.actions.AbstractActionPipelineProcessor;
import org.jboss.soa.esb.actions.ActionLifecycleException;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;
import org.jboss.soa.esb.actions.AbstractActionLifecycle;
import org.jboss.soa.esb.message.Body;
import org.jboss.soa.esb.http.HttpRequest;
import org.jboss.soa.esb.http.HttpResponse;
import org.apache.commons.httpclient.HttpStatus;
import org.jboss.resteasy.util.CaseInsensitiveMap;
import javax.ws.rs.core.MediaType;
import com.sun.jersey.api.client.ClientResponse;
import javax.xml.transform.Source;
import java.util.*;

import org.apache.log4j.Logger;

public class WADLAction extends AbstractActionLifecycle
{
  protected ConfigTree	_config;
	  
  public WADLAction(ConfigTree config) { _config = config; } 
  
  public Message noOperation(Message message) { return message; } 

  public Message displayMessage(Message message) throws Exception {		
		  logHeader();
		  Object order = message.getBody().get(Body.DEFAULT_LOCATION);
		  message.getBody().add( "user.ntfn.order", order );
		  System.out.println("Body [default]: " + message.getBody().get());
		  System.out.println("Body ['user.ntfn.order']: " + message.getBody().get("user.ntfn.order"));
		  System.out.println("Body ['user.ntfn.profile']: " + message.getBody().get("user.ntfn.profile"));
		  logFooter();
		  return message;         	
	}

  /**
   * Make Request to RESTful web service
   * 
   * This method was adapted from some code Dave had that took an incoming
   * HTTP request  and routed it.  This code expects no such incoming request
   * and instead makes and outgoing request to an arbitrary REST service, left
   * as an example for the reader -- my client wouldn't appreciate my leaving
   * a reference to their services in a code listing.  You therefore need to find
   * a RESTful service you can call and then supply that endpoint to the variable
   * named 'endpointUrl' in the code below.
   * 
   * @author Dave Siracusa
   * @param message
   * @return Message
   * @throws Exception
   */
  public Message process(Message message) throws Exception { // ActionProcessingException {
    try {
      System.out.println( "\n\n\n\n[[[[in WADLAction.process() ****]]]]" );
	  HttpRequest requestInfo = HttpRequest.getRequest(message);

	  if (requestInfo==null)
		// -=j=-: there was no incoming message if null
		System.out.println( "\n\n\n\n[[[[in WADLAction.process(): >>>> HttpRequest.getRequest(message) returned null -- no HttpRequest" );

      // -=j=-: get the payload from the incoming message -- the message that fired this action,
	  // it is *not* being routed from an external REST endpoint, it's on the JMS bus
      String restfulrequest=(String)message.getBody().get();
      System.out.println( "\n\n\n\n[[[[in WADLAction.process(): >>>> restfulrequest: "+restfulrequest );
      String method = null;
      // -=j=-:I added this, he's getting this from an incoming req which doesn't exist here
      String contentType = "application/xml";
      // -=j=-: if the params had to be put in encoded String form, here it is
      String endpointUrl = "https://foo.com/services/barService";
      ClientResponse response = null;
      Map<String, String[]> queryParamterMap = new HashMap<String, String[]>();

      System.out.println("WADLAction.process(): endpointUrl: [" + endpointUrl + "]");
      System.out.println("WADLAction.process(): method: [" + method + "]");
      System.out.println("WADLAction.process(): restfulrequest: [" + restfulrequest + "]");
      System.out.println("WADLAction.process(): requestInfo.getContentType(): [" +contentType+ "]");

      //if (requestInfo.getMethod().toLowerCase().contentEquals("post")) {
      if ( false ) {
        System.out.println("WADLAction.process(): -=j=-:::: hit the POST branch [was false so..?!?!?]" );
        JerseyClientSingleton.init();
        response = JerseyClientSingleton.post	(endpointUrl
											     //,MediaType.valueOf(requestInfo.getContentType())
											     ,MediaType.valueOf(contentType)
                                                 //,contentType
											     ,method
											     ,restfulrequest
											     );

      //} else if (requestInfo.getMethod().toLowerCase().contentEquals("get")) {
      // -=j=-: we're just going to do this one for now but can put these back in for POST, etc.
      } else if ( true ) {
    	method = "GET";
    	String op = "action";
        String[] opArray = { "getfoo" };
        String cacheParam = "bar";
        String[] cacheArray = { "27184283" };
        queryParamterMap.put(op, opArray);
        queryParamterMap.put(cacheParam, cacheArray);
        System.out.println( "WADLAction.process(): -=j=-: method: "+method );
    	JerseyClientSingleton.init();
    	response = JerseyClientSingleton.get	(endpointUrl
											    //,requestInfo.getQueryParams()
											    ,queryParamterMap
											    ,method
											    ,restfulrequest
											    //,(String)null
											    );
      } else {
        //throw new ActionProcessingException("Unknown method: " + requestInfo.getMethod());
    	//throw new Exception("Unknown method: " + requestInfo.getMethod());
    	System.out.println( "WADLAction.process(): -=j=-:::: hit the 'other method' [PUT,DELETE] branch [...?!?!?]" );
      }
      String entityResponse = response.getEntity(String.class);        
      message.getBody().add( "user.ntfn.profile", entityResponse );
      // -=j=-: put this in the default loc for the email writetap up next...
      message.getBody().add( entityResponse );
	} catch(Exception e) {
      System.out.println("Rest error"+e);
      message.getBody().add(e.toString());
    }
    return message;
  }

   public void exceptionHandler(Message message, Throwable exception) {
	   logHeader();
	   System.out.println("!ERROR!");
	   System.out.println(exception.getMessage());
	   System.out.println("For Message: ");
	   System.out.println(message.getBody().get());
	   logFooter();
   }
	
   private void logHeader() {
	   System.out.println("\n<<<<<<<<<<< WADLAction Header >>>>>>>>>>>>>>>>>");
   }
   private void logFooter() {
	   System.out.println("<<<<<<<<<<<<< WADLAction Footer >>>>>>>>>>>>>>>>>\n");
   }
    
	
}